
 

#include "xparameters.h"
#include "xio.h"
#include "xbasic_types.h"
#include "xstatus.h"
#include "stdio.h"
#include "lcd.h"

#define FLASH_START_ADDR 0x1F0000
#define MEM_SOURCE_START_ADDR 0x1F0000
#define MEM_START_ADDR 0x8000000

#define BITMAP_FILE_SIZE_OFFSET 0x0002
#define BITMAP_DATA_OFFSET 0x000A
#define BITMAP_WIDTH_OFFSET 0x0012
#define BITMAP_HEIGHT_OFFSET 0x0016
#define BITMAP_BITS_PER_PIXEL_OFFSET 0x001C
#define BITMAP_IMAGE_SIZE_OFFSET 0x0022

#define MEM_ROW_WIDTH 1024
#define MEM_DISPLAYED_ROW_WIDTH 640
#define MEM_DISPLAYED_HEIGHT 480

extern unsigned char verbose;


Xuint32 Read_Bitmap_Header_Bytes(Xuint32 FLASH_BASEADDR, Xuint32 offset)
{
Xuint32 Bitmap_Header_Data;
Xuint8 Flash_Data;

XIo_Out16 (FLASH_BASEADDR, 0xFF);
Flash_Data = XIo_In8(FLASH_BASEADDR + offset);
Bitmap_Header_Data = 0x00000000 | Flash_Data;
Flash_Data = XIo_In8(FLASH_BASEADDR + offset + 1);
Bitmap_Header_Data = Bitmap_Header_Data | (Flash_Data << 8);
Flash_Data = XIo_In8(FLASH_BASEADDR + offset + 2);
Bitmap_Header_Data = Bitmap_Header_Data | (Flash_Data << 16);
Flash_Data = XIo_In8(FLASH_BASEADDR + offset + 3);
Bitmap_Header_Data = Bitmap_Header_Data | (Flash_Data << 24);


}

Xuint32 Decode_display_bitmap()
{

Xuint16 Flash_Data;
Xuint8  Pixel_R;
Xuint8  Pixel_G;
Xuint8  Pixel_B;
Xuint32 Pixel_Data0, Pixel_Data1, Pixel_Data2;
Xuint32 temp;
Xuint32 Pixel_Data;
Xuint32 Bitmap_File_Size;
Xuint32 Bitmap_Data_Offset;
Xuint32 Bitmap_Width_Pixels;
Xuint32 Bitmap_Height_Pixels;
Xuint16 Bitmap_Bits_Per_Pixel;
Xuint32 Bitmap_Image_Size;
Xuint32 Bytes_Per_Row;
Xuint32 MEM_ADDR;
Xuint32 MEM_HIGHADDR;

Xuint32 i, j, k;


if (verbose) xil_printf ("\r\nDecoding bitmap");

//reset FLASH to read mode
XIo_Out16 (XPAR_FLASH_MEM0_BASEADDR, 0xFF);

Flash_Data = XIo_In16(XPAR_FLASH_MEM0_BASEADDR+FLASH_START_ADDR);
if (verbose) xil_printf("\r\nFile Type is 0x%X", Flash_Data);

Bitmap_File_Size = Read_Bitmap_Header_Bytes(XPAR_FLASH_MEM0_BASEADDR+FLASH_START_ADDR, BITMAP_FILE_SIZE_OFFSET);
//Bitmap_File_Size = XIo_In32(XPAR_FLASH_MEM0_BASEADDR+FLASH_START_ADDR+BITMAP_FILE_SIZE_OFFSET);
if (verbose) xil_printf("\r\nFile Size is 0x%X", Bitmap_File_Size);

Bitmap_Data_Offset = Read_Bitmap_Header_Bytes(XPAR_FLASH_MEM0_BASEADDR+FLASH_START_ADDR, BITMAP_DATA_OFFSET);
if (verbose) xil_printf("\r\nBitmap Data Offset is 0x%X", Bitmap_Data_Offset);

Bitmap_Width_Pixels = Read_Bitmap_Header_Bytes(XPAR_FLASH_MEM0_BASEADDR+FLASH_START_ADDR, BITMAP_WIDTH_OFFSET);
if (verbose) xil_printf("\r\nBitmap Width in pixels is %d", Bitmap_Width_Pixels);

Bitmap_Height_Pixels = Read_Bitmap_Header_Bytes(XPAR_FLASH_MEM0_BASEADDR+FLASH_START_ADDR, BITMAP_HEIGHT_OFFSET);
if (verbose) xil_printf("\r\nBitmap Height in pixels is %d", Bitmap_Height_Pixels);

Bitmap_Bits_Per_Pixel = XIo_In16(XPAR_FLASH_MEM0_BASEADDR + FLASH_START_ADDR + BITMAP_BITS_PER_PIXEL_OFFSET);
Bitmap_Bits_Per_Pixel = Bitmap_Bits_Per_Pixel >> 8;
if (verbose) xil_printf("\r\nBitmap Bits per pixel is %d", Bitmap_Bits_Per_Pixel);

Bitmap_Image_Size = Read_Bitmap_Header_Bytes(XPAR_FLASH_MEM0_BASEADDR+FLASH_START_ADDR, BITMAP_IMAGE_SIZE_OFFSET);
if (verbose) xil_printf("\r\nBitmap image size is %d", Bitmap_Image_Size);

Bytes_Per_Row = Bitmap_Image_Size / Bitmap_Height_Pixels;
if (verbose) xil_printf("\r\nBytes per row = %d", Bytes_Per_Row);

MEM_HIGHADDR = (MEM_ROW_WIDTH * MEM_DISPLAYED_HEIGHT) * 4;

XIo_Out32 (XPAR_XPS_TFT_0_SPLB_BASEADDR, XPAR_DDR2_SDRAM_MPMC_BASEADDR + MEM_START_ADDR);

MEM_ADDR = MEM_HIGHADDR - (MEM_ROW_WIDTH * 4);
if (verbose) xil_printf("\r\nImage Memory High address = 0x%X, Current Address = 0x%X", MEM_HIGHADDR, MEM_ADDR);

LCDSetLine(2);

for (i = 0;  i<(Bitmap_Height_Pixels * Bytes_Per_Row); i = i + Bytes_Per_Row)
{
	for (j = 0; j <Bytes_Per_Row; j = j + 3)
	{

		Pixel_B = XIo_In8(XPAR_FLASH_MEM0_BASEADDR+FLASH_START_ADDR
						   + Bitmap_Data_Offset + i + j);
		Pixel_G = XIo_In8(XPAR_FLASH_MEM0_BASEADDR+FLASH_START_ADDR
						   + Bitmap_Data_Offset + i + (j+1));
	    Pixel_R = XIo_In8(XPAR_FLASH_MEM0_BASEADDR+FLASH_START_ADDR
						   + Bitmap_Data_Offset + i + (j+2));

		Pixel_Data = ((0x00000000 | Pixel_R) << 16) | ((0x00000000 | Pixel_G) << 8) | (0x00000000 | Pixel_B);
		
		XIo_Out32(XPAR_DDR2_SDRAM_MPMC_BASEADDR + MEM_START_ADDR + MEM_ADDR, Pixel_Data);
		MEM_ADDR = MEM_ADDR + 4;

/*
		Pixel_Data0 = XIo_In32(XPAR_FLASH_MEM0_BASEADDR
						   + FLASH_START_ADDR + Bitmap_Data_Offset + i + j);
		Pixel_Data1 = XIo_In32(XPAR_FLASH_MEM0_BASEADDR
						   + FLASH_START_ADDR + Bitmap_Data_Offset + i + (j+4));
	    Pixel_Data2 = XIo_In32(XPAR_FLASH_MEM0_BASEADDR
						   + FLASH_START_ADDR + Bitmap_Data_Offset + i + (j+8));

		temp = Pixel_Data0;
		Pixel_Data = ((temp & 0xFF000000) >> 8); //R0
		temp = Pixel_Data0;
		Pixel_Data = Pixel_Data | ((temp & 0xFF00) >> 8) ;//B0
		temp = Pixel_Data0;
		Pixel_Data = Pixel_Data | ((temp & 0xFF) << 8) ;//G0
		XIo_Out32(XPAR_DDR2_SDRAM_MPMC_BASEADDR + MEM_START_ADDR + MEM_ADDR, Pixel_Data);
		MEM_ADDR = MEM_ADDR + 4;
				
		temp = Pixel_Data0;
		Pixel_Data = ((temp & 0x00FF0000) >> 16);//B1
		temp = Pixel_Data1;
		Pixel_Data = Pixel_Data | ((temp & 0xFF) << 16);//R1
		temp = Pixel_Data1;
		Pixel_Data = Pixel_Data | (temp & 0xFF00);//G1
		XIo_Out32(XPAR_DDR2_SDRAM_MPMC_BASEADDR + MEM_START_ADDR + MEM_ADDR, Pixel_Data);
		MEM_ADDR = MEM_ADDR + 4;
		
		temp = Pixel_Data1;
		Pixel_Data = ((temp & 0x00FF0000) >> 8);//G2
		temp = Pixel_Data1;
		Pixel_Data = Pixel_Data | ((temp & 0xFF000000) >> 24);//B2
		temp = Pixel_Data2;
		Pixel_Data = Pixel_Data | ((temp & 0x0000FF00) << 8);//R2
		XIo_Out32(XPAR_DDR2_SDRAM_MPMC_BASEADDR + MEM_START_ADDR + MEM_ADDR, Pixel_Data);
		MEM_ADDR = MEM_ADDR + 4;
		
		temp = Pixel_Data2;
		Pixel_Data = (temp & 0x00FF0000);//R3
		temp = Pixel_Data2;
		Pixel_Data = Pixel_Data | ((temp & 0xFF000000) >> 16);//G3
		temp = Pixel_Data2;
		Pixel_Data = Pixel_Data | (temp & 0xFF);//B3
		XIo_Out32(XPAR_DDR2_SDRAM_MPMC_BASEADDR + MEM_START_ADDR + MEM_ADDR, Pixel_Data);
		MEM_ADDR = MEM_ADDR + 4;
*/
	}
	//if (verbose) xil_printf("\r\nCurrent MEM_ADDR value is 0x%X", MEM_ADDR);
	MEM_ADDR = MEM_ADDR - ((MEM_DISPLAYED_ROW_WIDTH + MEM_ROW_WIDTH) * 4);	
	//if (verbose) xil_printf("\r\nMEM_ADDR value is 0x%X", MEM_ADDR);
	
	if (!(i&0xFFF))
	{
		LCDPrintChar('*');
	}


}

if (verbose) xil_printf("\r\nMEM_ADDR value is 0x%X", MEM_ADDR);

//XIo_Out32 (XPAR_XPS_TFT_0_SPLB_BASEADDR, XPAR_DDR2_SDRAM_MPMC_BASEADDR + MEM_START_ADDR);

}

