
// Located in: microblaze_0/include/xparameters.h
#include "xparameters.h"

#include "stdio.h"

#include "xintc.h"
#include "intc_header.h"
#include "xuartns550_l.h"
#include "uartns550_header.h"
#include "xuartns550.h"
#include "xbasic_types.h"
#include "xgpio.h"
#include "xlltemac.h"
#include "xlltemac_example.h"
#include "lltemac_header.h"
#include "lltemac_intr_header.h"
#include "xlldma.h"
#include "uartlite_header.h"
#include "sleep.h"

#define HARD_RESET 1
#define SOFT_RESET 2

#define GMII_MODE 0
#define SGMII_MODE 1

#define LED_NORMAL  0x0
#define LED_ON      0x3
#define LED_OFF     0x2
#define LED_BLINK   0x1

#define PHY_INTERNAL_LOOPBACK 1
#define PHY_LOOPBACK_STUB 0

extern int PhyAddr;

extern unsigned char verbose;
   


int DetectPhy (XLlTemac * TemacInstancePtr){

int err;
u16 Id_Reg1, Id_Reg2, PhyReg;
Xuint32 Marvell_OUI;
u8 Device_num;
u8 Revision_num;

#define PHY_DETECT_REG  1

/* Mask used to verify certain PHY features (or register contents)
 * in the register above:
 *  0x1000: 10Mbps full duplex support
 *  0x0800: 10Mbps half duplex support
 *  0x0008: Auto-negotiation support
 */
#define PHY_DETECT_MASK 0x1808

err = 0;

    for (PhyAddr = 31; PhyAddr >= 0; PhyAddr--) {
        XLlTemac_PhyRead(TemacInstancePtr, PhyAddr, PHY_DETECT_REG,
                 &PhyReg);

        if ((PhyReg != 0xFFFF) &&
            (PhyReg & PHY_DETECT_MASK == PHY_DETECT_MASK)) {
            /* Found a valid PHY address */
         if (verbose) xil_printf ("\r\nPHY detected address is 0x%X", PhyAddr);
         break;
        }
    }
   
   if (!(PhyAddr == 7 )){
        if (verbose) xil_printf ("\r\nERROR in Phy detect address, expected: 7, got: %d", PhyAddr);
        err++;
   }

//read the phy identifier

   XLlTemac_PhyRead(TemacInstancePtr, PhyAddr, 2, &Id_Reg1);
   //xil_printf ("\r\n PHY Identifier Register 1 is 0x%X", Id_Reg1);

   XLlTemac_PhyRead(TemacInstancePtr, PhyAddr, 3, &Id_Reg2);
   //xil_printf ("\r\n PHY Identifier Register 2 is 0x%X", Id_Reg2);
 
   Marvell_OUI = (Id_Reg1 << 6) | ((Id_Reg2 & 0xFC00) >> 10);
   Device_num = ((Id_Reg2 & 0x03F0) >> 4);
   Revision_num = (Id_Reg2 & 0x000F);
    
   
   if (verbose) xil_printf ("\r\n OUI is 0x%06X", Marvell_OUI);
   if (verbose) xil_printf ("\r\nDevice ID is 0x%02X", Device_num);
   if (verbose) xil_printf ("\r\nRevision Number is 0x%X", Revision_num);

   if (!(Marvell_OUI == 0x005043)){
        if (verbose) xil_printf ("\r\nERROR: Phy identifier does not match Marvell OUI");
        err++;
   }

   if (!(Device_num == 0x0C)){
        if (verbose) xil_printf ("\r\nERROR: Phy Device is not 88E111");
        err++;
   }

    return err;
   
}

int ResetPhy (XLlTemac * TemacInstancePtr, int ResetType){

Xuint32 Reset_Status;
int i;
u16 PhyReg;

i=0;

if (ResetType == HARD_RESET){

      if (verbose) xil_printf ("\r\nWriting reset data...");
       
      XIo_Out32 (XPAR_PHY_RESET_COMPONENT_0_BASEADDR, 0x01);
      if (verbose) xil_printf ("\r\nReading status data..");      
      Reset_Status = XIo_In32(XPAR_PHY_RESET_COMPONENT_0_BASEADDR);
      if (verbose) xil_printf ("\r\nStatus data = 0x%X", Reset_Status);
      while (Reset_Status)
      {
         Reset_Status = XIo_In32(XPAR_PHY_RESET_COMPONENT_0_BASEADDR);
         if (verbose) xil_printf ("\r\nStatus data = 0x%X", Reset_Status);
      }
      
}
        
else if (ResetType == SOFT_RESET){    
   XLlTemac_PhyRead(TemacInstancePtr, PhyAddr, 0, &PhyReg);
   PhyReg = PhyReg | 0x8000;
   XLlTemac_PhyWrite(TemacInstancePtr, PhyAddr, 0,    PhyReg);
   
   XLlTemac_PhyRead(TemacInstancePtr, PhyAddr, 0, &PhyReg);
   while (PhyReg & 0x8000)
   {
      XLlTemac_PhyRead(TemacInstancePtr, PhyAddr, 0, &PhyReg);
      i++;
      usleep(100);
      if (i==5000) return XST_FAILURE;
   }
}

}

int CheckPhyInitialConfig (XLlTemac * TemacInstancePtr, int PhyMode){

u16 ANEG;
u16 ENA_XC, DIS_125;
u16 HWCFG;
u16 DIS_FC, DIS_SLEEP;
u16 INT_POL, Ohm_75_50;

u16 PhyRegs[32];
u16 * p_PhyRegs;
int i;

int err;

err = 0;

// read all the PHY registers

p_PhyRegs = PhyRegs;

for (i=0; i<32; i++){
    XLlTemac_PhyRead(TemacInstancePtr, PhyAddr, i, p_PhyRegs); 
    p_PhyRegs++;
}

//check the register values

// GMII Mode first
if (PhyMode == GMII_MODE) {
   
//ANEG should be 1111 in GMII mode
if ( ((PhyRegs [0] & 0x1140) == 0x1140) && //Register 0 bits 12, 8 and 6 should be 1
     ((PhyRegs [0] & 0x2000) == 0     ) && //Register 0 bit 13 should be 0
     ((PhyRegs [4] & 0x01E0) == 0x01E0) && //Register 4 bits 8,7,6 and 5 should be 1
     ((PhyRegs [9] & 0x0300) == 0x0300) && //Register 9 bits 9 and 8 should be 1
     ((PhyRegs [9] & 0x1C00) == 0     ) && //Register 9 bits 12, 11 and 10 should be 0
     ((PhyRegs[16] & 0x0060) == 0x0060)) //Register 16 bits 6:5 should be 1
    ANEG=15;
else ANEG = 0;

if (ANEG != 15 ){
    err++;
    if (verbose) xil_printf ("\r\nERROR, ANEG is different from 1111");
}   
            
//ENA_XC should be 1 in GMII mode
if (((PhyRegs [16] & 0x0060) == 0x0060)) ENA_XC = 1; //ENA_XC is tested by looking into
                                                     //register 16 bits 6:5, should be 11
else ENA_XC = 0;

if (!(ENA_XC)){
    err++;
    if (verbose) xil_printf ("\r\nERROR, ENA_XC is 0, expected: 1");
}   

//DIS_125 should be 1 in GMII mode
if (((PhyRegs [16] & 0x0010) == 0x0010)) DIS_125 = 1; //DIS_125 is Register 16 bit 1
else DIS_125 = 0;

if (!(DIS_125)){
    err++;
    if (verbose) xil_printf ("\r\nERROR, DIS_125 is 0, expected: 1");
}   

//HWCFG should be 1111 in GMII mode
HWCFG = (PhyRegs [27] & 0x0F);

if (HWCFG != 15){
    err++;
    if (verbose) xil_printf ("\r\nERROR, incorrect HWCFG mode, expected:0x0F, got: 0x%X", HWCFG);
}   

//DIS_FC should be 1 in GMII mode
if (((PhyRegs [27] & 0x8000) == 0x8000)) DIS_FC = 1; //DIS_FC is Register 27 bit 15
else DIS_FC = 0;

if (!(DIS_FC)){
    err++;
    if (verbose) xil_printf ("\r\nERROR, DIS_FC is 0, expected: 1");
}   

//DIS_SLEEP should be 1 in GMII mode
if (((PhyRegs [16] & 0x0300) == 0)) DIS_SLEEP = 1; //Register 16 bits 9 and 8 should be 0
else DIS_SLEEP = 0;

if (!(DIS_SLEEP)){
    err++;
    if (verbose) xil_printf ("\r\nERROR, DIS_SLEEP is 0, expected: 1");
}   

//INT_POL should be 1 in GMII mode
if (((PhyRegs [27] & 0x0400) == 0x00400)) INT_POL = 1; //INT_POL is Register 16 bit 10
else INT_POL = 0;

if (!(INT_POL)){
    err++;
    if (verbose) xil_printf ("\r\nERROR, INT_POL is 0, expected: 1");
}   

//Ohm_75_50 should be 0 in GMII mode
if (((PhyRegs [26] & 0x0060) == 0)) Ohm_75_50 = 0; // Ohm_75_50 is determined by Register 26 bits 6 and 5
else Ohm_75_50 = 1;
    
if (Ohm_75_50){
    err++;
    if (verbose) xil_printf ("\r\nERROR, Ohm_75_50 is 1, expected: 0");
}   

if (err){
	if (verbose)
	{
	    xil_printf("\r\nPhy Hardware Configuration error");
	    xil_printf("\r\nPlease check if:");
	    xil_printf("\r\n     J22 and J23 are set between 1 and 2 and J24 is open");
	    xil_printf("\r\n     Check the voltages on the CONFIG lines");
	    xil_printf("\r\nBeginning Register dump :");
	    for (i=0; i<32; i++){
	    xil_printf("\r\n    REG %d: 0x%04X", i, PhyRegs[i]);
	 	}
    }
}
else if (verbose) xil_printf("\r\nPhy Hardware configured correctly in GMII mode");

return err;
}

else if (PhyMode == SGMII_MODE){

//ANEG should be 1111 in SGMII mode
if ( ((PhyRegs [0] & 0x1140) == 0x1140) && //Register 0 bits 12, 8 and 6 should be 1
     ((PhyRegs [0] & 0x2000) == 0     ) && //Register 0 bit 13 should be 0
     ((PhyRegs [4] & 0x01E0) == 0x01E0) && //Register 4 bits 8,7,6 and 5 should be 1
     ((PhyRegs [9] & 0x0300) == 0x0300) && //Register 9 bits 9 and 8 should be 1
     ((PhyRegs [9] & 0x1C00) == 0     ) && //Register 9 bits 12, 11 and 10 should be 0
     ((PhyRegs[16] & 0x0060) == 0x0060)) //Register 16 bits 6:5 should be 1
    ANEG=15;
else ANEG = 0;

if (ANEG != 15 ){
    err++;
    if (verbose) xil_printf ("\r\nERROR, ANEG is different from 1111");
}   
            
//ENA_XC should be 1 in SGMII mode
if (((PhyRegs [16] & 0x0060) == 0x0060)) ENA_XC = 1; //ENA_XC is tested by looking into
                                                     //register 16 bits 6:5, should be 11
else ENA_XC = 0;

if (!(ENA_XC)){
    err++;
    if (verbose) xil_printf ("\r\nERROR, ENA_XC is 0, expected: 1");
}   

//DIS_125 should be 1 in SGMII mode
if (((PhyRegs [16] & 0x0010) == 0x0010)) DIS_125 = 1; //DIS_125 is Register 16 bit 1
else DIS_125 = 0;

if (!(DIS_125)){
    err++;
    if (verbose) xil_printf ("\r\nERROR, DIS_125 is 0, expected: 1");
}   

//HWCFG should be 0100 in SGMII mode
HWCFG = (PhyRegs [27] & 0x0F);

if (HWCFG != 4){
    err++;
    if (verbose) xil_printf ("\r\nERROR, incorrect HWCFG mode, expected:0x04, got: 0x%X", HWCFG);
}   

//DIS_FC should be 1 in SGMII mode
if (((PhyRegs [27] & 0x8000) == 0x8000)) DIS_FC = 1; //DIS_FC is Register 27 bit 15
else DIS_FC = 0;

if (!(DIS_FC)){
    err++;
    if (verbose) xil_printf ("\r\nERROR, DIS_FC is 0, expected: 1");
}   

//DIS_SLEEP should be 1 in SGMII mode
if (((PhyRegs [16] & 0x0300) == 0)) DIS_SLEEP = 1; //Register 16 bits 9 and 8 should be 0
else DIS_SLEEP = 0;

if (!(DIS_SLEEP)){
    err++;
    if (verbose) xil_printf ("\r\nERROR, DIS_SLEEP is 0, expected: 1");
}   

//INT_POL should be 1 in SGMII mode
if (((PhyRegs [27] & 0x0400) == 0x00400)) INT_POL = 1; //INT_POL is Register 16 bit 10
else INT_POL = 0;

if (!(INT_POL)){
    err++;
    if (verbose) xil_printf ("\r\nERROR, INT_POL is 0, expected: 1");
}   

//Ohm_75_50 should be 0 in SGMII mode
if (((PhyRegs [26] & 0x0060) == 0)) Ohm_75_50 = 0; // Ohm_75_50 is determined by Register 26 bits 6 and 5
else Ohm_75_50 = 1;
    
if (Ohm_75_50){
    err++;
    if (verbose) xil_printf ("\r\nERROR, Ohm_75_50 is 1, expected: 0");
}   

if (err){
	if (verbose)
	{
	    xil_printf("\r\nPhy Hardware Configuration ERROR");
	    xil_printf("\r\nPlease check if:");
	    xil_printf("\r\n     J22 and J23 are set between 2 and 3 and J24 is open");
	    xil_printf("\r\n     Check the voltages on the CONFIG lines");
	    xil_printf("\r\nBeginning Register dump :");
	    xil_printf("\r\n    REG 0 : 0x%04X", PhyRegs[0]);
	    xil_printf("\r\n    REG 4 : 0x%04X", PhyRegs[4]);
	    xil_printf("\r\n    REG 9 : 0x%04X", PhyRegs[9]);
	    xil_printf("\r\n    REG 16: 0x%04X", PhyRegs[16]);
	    xil_printf("\r\n    REG 26: 0x%04X", PhyRegs[26]);
	    xil_printf("\r\n    REG 27: 0x%04X", PhyRegs[27]);
	}
}
else if (verbose) xil_printf("\r\nPhy Hardware configurred correctly in SGMII mode");

return err;
}
 
}

int IndividualSetPhyLeds(XLlTemac * TemacInstancePtr, u8 LedNum, u8 LedMode){
//set the Phy LEDs according to the bits of LedNum and LedMode
//LedMode = 0x0 - Normal operation
//LedMode = 0x1 - LED blinking
//LedMode = 0x2 - LED off
//LedMode = 0x3 - LED on
//bit 0 LedNum - LED_TX
//bit 1 Lednum - LED_RX
//bit 2 LedNum - LED_LINK_1000
//bit 3 LedNum - LED_LINK_100
//bit 4 LedNum - LED_LINK_10
//bit 5 LedNum - LED_DUPLEX


u16 PhyReg;
u16 temp;
Xuint32 i;

   XLlTemac_PhyRead(TemacInstancePtr, PhyAddr, 25, &PhyReg);
   PhyReg = PhyReg & 0x3FFF;


   if (!((LedMode == LED_NORMAL) ||
       (LedMode == LED_ON    ) ||
       (LedMode == LED_OFF   ) ||
       (LedMode == LED_BLINK )))
   {
	    if (verbose) xil_printf ("\r\nLED Mode incorrect");
	    return XTEST_FAILED;
   }

	for (i=0; i<7; i++)
	{
	 	if (LedNum & (1 << i))
	 	{
		  	temp = (0xFFFF << (2*(i+1))) | (0xFFFF >> (2*(8-i)));
		   	PhyReg = PhyReg & temp;
		   	PhyReg = PhyReg | (LedMode << (2*i));
	 	}
	}

   XLlTemac_PhyWrite(TemacInstancePtr, PhyAddr, 25, PhyReg);
   return XST_SUCCESS;
}



int PhyVirtualCableTest (XLlTemac * TemacInstancePtr) {
//returns:
// 0 if cable test completed successfully
// 1 if cable open or short found on some of the lines
// 2 if cable tester timeout encountered
// 3 if Virtual cable test internal error

u16 PhyReg;
int err, open_found, short_found, test_fail;
u16 MDI0_Test, MDI1_Test, MDI2_Test, MDI3_Test;
int i;

err = 0;
open_found = 0;
short_found = 0;
test_fail = 0;
i=0;

   //first start testing by writing 1 into the MDI[0] register (register 28 page 0) bit 15
   XLlTemac_PhyRead(TemacInstancePtr, PhyAddr, 28, &PhyReg);
   PhyReg = PhyReg | 0x8000;
   XLlTemac_PhyWrite(TemacInstancePtr, PhyAddr, 28,   PhyReg);
   XLlTemac_PhyRead(TemacInstancePtr, PhyAddr, 28, &PhyReg);
   while (PhyReg & 0x8000)
   {
      XLlTemac_PhyRead(TemacInstancePtr, PhyAddr, 28, &PhyReg);
         i++;
         usleep(100);
         if (i==5000) {
                        if (verbose) xil_printf("\r\nERROR: Cable Test Timeout");
                        return 2;
                        }
   }
    
   //read MDI[0] register (register 28 page 0) for test results
   XLlTemac_PhyRead(TemacInstancePtr, PhyAddr, 28, &MDI0_Test);

   PhyReg = ((MDI0_Test & 0x6000) >> 13);
   switch (PhyReg){
    case 1:     short_found = 1;
                if (verbose) xil_printf ("\r\nERROR: Short in MDI[0] cable pair");
                break;
    case 2:     open_found = 1;
                if (verbose) xil_printf ("\r\nERROR: open in MDI[0] cable pair");
                break;
    case 3:     test_fail = 1;
                if (verbose) xil_printf ("\r\nERROR: Virtual Cable Test internal error");
                break;
    default:   break;
   }
   //read MDI[1] register by writing 1 to the page address (register 22)
   XLlTemac_PhyWrite(TemacInstancePtr, PhyAddr, 22,   1);
   XLlTemac_PhyRead(TemacInstancePtr, PhyAddr, 28, &MDI1_Test);
   
   PhyReg = ((MDI1_Test & 0x6000) >> 13);
   switch (PhyReg){
    case 1:     short_found = 1;
                if (verbose) xil_printf ("\r\nERROR: Short in MDI[1] cable pair");
                break;
    case 2:     open_found = 1;
                if (verbose) xil_printf ("\r\nERROR: open in MDI[1] cable pair");
                break;
    case 3:     test_fail = 1;
                if (verbose) xil_printf ("\r\nERROR: Virtual Cable Test internal error");
                break;
    default:   break;
   }
    
   //read MDI[2] register by writing 2 to the page address (register 22)
   XLlTemac_PhyWrite(TemacInstancePtr, PhyAddr, 22,   2);
   XLlTemac_PhyRead(TemacInstancePtr, PhyAddr, 28, &MDI2_Test);

   PhyReg = ((MDI2_Test & 0x6000) >> 13);
   switch (PhyReg){
    case 1:     short_found = 1;
                if (verbose) xil_printf ("\r\nERROR: Short in MDI[2] cable pair");
                break;
    case 2:     open_found = 1;
                if (verbose) xil_printf ("\r\nERROR: open in MDI[2] cable pair");
                break;
    case 3:     test_fail = 1;
                if (verbose) xil_printf ("\r\nERROR: Virtual Cable Test internal error");
                break;
    default:   break;
   }

   //read MDI[3] register by writing 3 to the page address (register 22)
   XLlTemac_PhyWrite(TemacInstancePtr, PhyAddr, 22,   3);
   XLlTemac_PhyRead(TemacInstancePtr, PhyAddr, 28, &MDI3_Test);

   PhyReg = ((MDI3_Test & 0x6000) >> 13);
   switch (PhyReg){
    case 1:     short_found = 1;
                if (verbose) xil_printf ("\r\nERROR: Short in MDI[3] cable pair");
                break;
    case 2:     open_found = 1;
                if (verbose) xil_printf ("\r\nERROR: open in MDI[3] cable pair");
                break;
    case 3:     test_fail = 1;
                if (verbose) xil_printf ("\r\nERROR: Virtual Cable Test internal error");
                break;
    default:   break;
   }

   //reset the page register
   XLlTemac_PhyWrite(TemacInstancePtr, PhyAddr, 22,   0);
   
   if (test_fail) err = 3;
   if ((open_found) || (short_found)) err = 1;
   
   if (err) if (verbose) xil_printf ("\r\nErrors found during cable test");

if (verbose)
{
   xil_printf("\r\nBeginning Register dump :");
   xil_printf ("\r\nMDI[0] register value: 0x%04X", MDI0_Test);
   xil_printf ("\r\nMDI[1] register value: 0x%04X", MDI1_Test);
   xil_printf ("\r\nMDI[2] register value: 0x%04X", MDI2_Test);
   xil_printf ("\r\nMDI[3] register value: 0x%04X", MDI3_Test);
}
   return err;
}

int GetPhyLinkStatus (XLlTemac * TemacInstancePtr, int * PhyLink){
// returns 0 if the speed was resolved and link is up
// returns 1 if the speed was resolved and link is down
// returns 2 if the speed was not resolved 
// PhyLink is a pointer to an integer array with 3 elements
// 0: Link Status
// 1: Speed
// 2: Duplex

int * PhyLinkData;
u16 PhyReg;

int LinkStatus;
int Speed;
int i;

i = 0;
PhyLinkData = PhyLink;

   //Check first linka and speed are resolved
   // See first if Serial Interface AutoNegotiation is in bypass status

   XLlTemac_PhyRead(TemacInstancePtr, PhyAddr, 27, &PhyReg);
   if (!(PhyReg & 0x0800)) { //if Autonegotiation is not bypassed 
    XLlTemac_PhyRead(TemacInstancePtr, PhyAddr, 17, &PhyReg);
    //check if the speed and duplex are resolved or if there is link
    while (!(PhyReg & 0x0800)){
        XLlTemac_PhyRead(TemacInstancePtr, PhyAddr, 17, &PhyReg);
        i++;
        usleep(100);
        if (i==1000)
        {
               if (PhyReg & 0x0400) break; // if there is link and the speed is not resolved
                                            // it means that the negotiated speed is different from what
                                            // the PHY advertised, for example the speed is 100 but the PHY advertised 1000T
               else {
                 if (verbose) xil_printf("\r\nERROR: Speed and Duplex resolved timeout");
                 if (verbose) xil_printf("\r\nStatus Register value: 0x%04X", PhyReg);
                 return 2;
                      }
        }                   
    }
    
   }

   XLlTemac_PhyRead(TemacInstancePtr, PhyAddr, 17, &PhyReg);
   
   //read the link status;
   LinkStatus = (PhyReg & 0x0400) >> 10;
   
   
   *PhyLinkData = LinkStatus;
   PhyLinkData++;
   
   //read the actual speed
   Speed = ((PhyReg & 0xC000) >> 14);
   
   switch (Speed) {
   case 0: *PhyLinkData = 10;
             break;
   case 1: *PhyLinkData = 100;
             break;
   case 2: *PhyLinkData = 1000;
             break;
   default: break;
   }
   PhyLinkData++;
   
   //read the actual duplex value 
   *PhyLinkData = ((PhyReg & 0x2000) >> 13);
   
   if (LinkStatus) return 0;
   else return 1;
}


int RestartPhyAutonegotiation (XLlTemac * TemacInstancePtr){
// Enables AutoNegotiation if not enabled restarts it 
// Returns XST_SUCCESS if autonegotiation was completed successfully
// Returns XST_FAILURE if autonegotiation timeout
       
u16 PhyReg;
int i;

XStatus Status;

i = 0;

    //check first if AutoNegotiation is enabled
    //if not, enable it;
    XLlTemac_PhyRead(TemacInstancePtr, PhyAddr, 0, &PhyReg);
    if (!(PhyReg & 0x1000)){
       if (verbose)  xil_printf("\r\nAutonegotiation not enabled, enabling...");
        PhyReg = PhyReg | 0x1000;
        XLlTemac_PhyWrite(TemacInstancePtr, PhyAddr, 0, PhyReg);
        // a software reset has to be performed
        Status = ResetPhy (TemacInstancePtr, SOFT_RESET);   
    }
    
    //restart AutoNegotiation by writing 1 to Register 0 bit 9
    XLlTemac_PhyRead(TemacInstancePtr, PhyAddr, 0, &PhyReg);
    PhyReg = PhyReg | 0x0200;
    XLlTemac_PhyWrite(TemacInstancePtr, PhyAddr, 0, PhyReg);

    XLlTemac_PhyRead(TemacInstancePtr, PhyAddr, 1, &PhyReg);
    while (!(PhyReg & 0x0020)){
       XLlTemac_PhyRead(TemacInstancePtr, PhyAddr, 1, &PhyReg);
       i++;
       usleep(100);
       if (i==1000) {
                       if (verbose) xil_printf("\r\nERROR: Autonegotiation Timeout");
                       return XST_FAILURE;
                        }       
    }
    
    return XST_SUCCESS;
}

int SetPhyLoopbackMode (XLlTemac * TemacInstancePtr, int Loopback_Speed, int Loopback_Mode){
//Loopback_Mode can be PHY_INTERNAL_LOOPBACK (1) or PHY_LOOPBACK_STUB (0)
//Loopback_Speed can be 10, 100 or 1000;

u16 PhyReg0;
u16 PhyReg;
u16 Speed_Bit_Set;
u16 Speed_Bit_Clear;
u16 Loopback;

XStatus Status;

if (Loopback_Mode > 1){
        if (verbose) xil_printf("\r\nIncorrect Loopback_Mode");
        return XST_FAILURE;
}

if ((Loopback_Speed !=   10) &&
    (Loopback_Speed !=  100) &&
    (Loopback_Speed != 1000)) {
        if (verbose) xil_printf("\r\nLoopback_Speed not 10, 100, or 1000 Mbps");
        return XTEST_FAILED;
}

//Disable AutoNegotiation first
   XLlTemac_PhyRead(TemacInstancePtr, PhyAddr, 0, &PhyReg);
   PhyReg = PhyReg & 0xEFFF;
   XLlTemac_PhyWrite(TemacInstancePtr, PhyAddr, 0, PhyReg);
   // a software reset has to be performed
   Status = ResetPhy (TemacInstancePtr, SOFT_RESET);   
   

Loopback = (Loopback_Mode << 14);

if (Loopback_Speed == 10){
    Speed_Bit_Set = 0;
    Speed_Bit_Clear = 0xDFBF; //clear bits 13 and 6
    XLlTemac_PhyRead(TemacInstancePtr, PhyAddr, 0, &PhyReg);
    PhyReg = (PhyReg | Speed_Bit_Set) & Speed_Bit_Clear;
    XLlTemac_PhyWrite(TemacInstancePtr, PhyAddr, 0, PhyReg);
    Status = ResetPhy (TemacInstancePtr, SOFT_RESET);
    if (Loopback) {
           XLlTemac_PhyRead(TemacInstancePtr, PhyAddr, 0, &PhyReg);
           PhyReg = PhyReg | Loopback;
           XLlTemac_PhyWrite(TemacInstancePtr, PhyAddr, 0, PhyReg);
    }
}

if (Loopback_Speed == 100){
    Speed_Bit_Set = 0x2000;
    Speed_Bit_Clear = 0xFFBF; //set bit 13 and clear bit 6
    XLlTemac_PhyRead(TemacInstancePtr, PhyAddr, 0, &PhyReg);
    PhyReg = (PhyReg | Speed_Bit_Set) & Speed_Bit_Clear;
    XLlTemac_PhyWrite(TemacInstancePtr, PhyAddr, 0, PhyReg);
    Status = ResetPhy (TemacInstancePtr, SOFT_RESET);
    if (Loopback) {
           XLlTemac_PhyRead(TemacInstancePtr, PhyAddr, 0, &PhyReg);
           PhyReg = PhyReg | Loopback;
           XLlTemac_PhyWrite(TemacInstancePtr, PhyAddr, 0, PhyReg);
    }
}

if (Loopback_Speed == 1000){
    if (!(Loopback)) { //configure the phy for external stub loopback in 1000Mbps mode
            PhyReg = 0;
            XLlTemac_PhyWrite(TemacInstancePtr, PhyAddr, 18, PhyReg); //disable interrupts
            XLlTemac_PhyRead(TemacInstancePtr, PhyAddr, 9, &PhyReg);
            PhyReg = PhyReg | 0x1800;
            XLlTemac_PhyWrite(TemacInstancePtr, PhyAddr, 9, PhyReg); //force Master
            Status = ResetPhy (TemacInstancePtr, SOFT_RESET);//soft reset to update
    
            XLlTemac_PhyWrite(TemacInstancePtr, PhyAddr, 29, 7); //select page 7 of Register 30
            XLlTemac_PhyRead(TemacInstancePtr, PhyAddr, 30, &PhyReg);
            PhyReg = PhyReg | 0x0008;
            XLlTemac_PhyWrite(TemacInstancePtr, PhyAddr, 30, PhyReg); //force gigabit mode
    
            XLlTemac_PhyWrite(TemacInstancePtr, PhyAddr, 29, 16); //select page 16 of Register 30
            XLlTemac_PhyRead(TemacInstancePtr, PhyAddr, 30, &PhyReg);
            PhyReg = PhyReg | 0x0002;
            XLlTemac_PhyWrite(TemacInstancePtr, PhyAddr, 30, PhyReg); //enable Gigabit stub loopback

            XLlTemac_PhyWrite(TemacInstancePtr, PhyAddr, 29, 18); //select page 18 of Register 30
            XLlTemac_PhyRead(TemacInstancePtr, PhyAddr, 30, &PhyReg);
            PhyReg = PhyReg | 0x0001;
            XLlTemac_PhyWrite(TemacInstancePtr, PhyAddr, 30, PhyReg); //disable Next Canceller
        }
    else {
            Speed_Bit_Set = 0x0040;
            Speed_Bit_Clear = 0xDFFF; //set bit 6 and clear bit 13
            XLlTemac_PhyRead(TemacInstancePtr, PhyAddr, 0, &PhyReg);
            PhyReg = (PhyReg | Speed_Bit_Set) & Speed_Bit_Clear;
            XLlTemac_PhyWrite(TemacInstancePtr, PhyAddr, 0, PhyReg);
            Status = ResetPhy (TemacInstancePtr, SOFT_RESET);
            XLlTemac_PhyRead(TemacInstancePtr, PhyAddr, 0, &PhyReg);//set the internal loopback
            PhyReg = PhyReg | Loopback;
            XLlTemac_PhyWrite(TemacInstancePtr, PhyAddr, 0, PhyReg);
    }
    
}

return XST_SUCCESS;
}



